##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
"""DublinCore mix-in"""

__version__='$Revision: 1.4 $'[11:-2]

import Globals

class DublinCore:
    """Mix-in class which provides Dublin Core methods"""

    __ac_permissions__=(
        ('View',
         ('Creator','Date', 'Title', 'Description', 'Subject', 'Identifier',
          'Type'),
         ('Owner','Manager','Reviewer')),
        )

    def Title(self):
        "Dublin Core element - resource name"
        return getattr(self, 'title', 'Untitled')

    def Creator(self):
        "Dublin Core element - resource creator"
        if hasattr(self, 'get_local_roles'):
            for user, roles in self.get_local_roles():
                if 'Owner' in roles:
                    return(user)
        return 'No owner'
    Contributor = Creator
    
    def Date(self):
        "Dublin Core element - effective date"
        # Return effective_date (as date object) if set,
        # modification date otherwise
        if getattr(self, 'effective_date', None):
            return self.effective_date
        return self.bobobase_modification_time()

    def Description(self):
        "Dublin Core element - summary"
        # subclasses should override this method
        return getattr(self, 'description', '')

    def Subject(self):
        "Dublin Core element - Topical keywords"
        subject = getattr(self, 'subject', [])
        if type(subject) == type([]): return subject
        if type(subject) == type(()): return list(subject)
        if type(subject) == type(''): return [subject]
        raise ValueError, "subject of unsupported type"

    def Type(self):
        "Dublin Core element - Object type"
        return getattr(self, 'meta_type', 'Unknown type')

    def Identifier(self):
        "Dublin Core element - Object ID"
        return self.id


Globals.default__class_init__(DublinCore)
