##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
"""Folder manageable through portal interface"""

__version__='$Revision: 1.17 $'[11:-2]

import Globals

from OFS.Folder import Folder
from Globals import HTMLFile


class PortalFolder(Folder):
    "Implements portal content management"

    __ac_permissions__=(
        ('View management screens',
         ('portal_contents','portal_cut','portal_copy','portal_paste',
          'portal_renameForm','portal_rename'),
         ('Owner','Manager')),
        ('Delete objects', ('portal_delete',), ('Owner','Manager')),
        ('Add Portal Folders',
         ('manage_addPortalFolder',),
         ('Owner','Manager')),
        )

    # Each module defining a Portal subobject registers with this meta_types
    meta_types = (
        {'name': 'Portal Folder',
         'action': 'manage_addPortalFolderForm',
         'permission': 'Add Folders'},
        )
    
    manage_addPortalFolderForm=HTMLFile('dtml/folderAdd', globals())

    def manage_addPortalFolder(self, id, title='',
                               REQUEST=None):
        """Add a new PortalFolder object with id *id*.
        """
        ob=PortalFolder()
        ob.id=id
        ob.title=title
        self._setObject(id, ob)
        if REQUEST is not None:
            return self.portal_contents(
                self, REQUEST, portal_status_message="Folder added")

    # Portal UI Manager interface

    portal_contents = HTMLFile('dtml/portal_contents',globals())
    
    def portal_cut(self, ids, REQUEST=None):
        "Wrapper around manage_cutObjects, avoids management interface"
        self.manage_cutObjects(ids, REQUEST)
        return self.portal_contents(self, REQUEST,
                                    portal_status_message="Cut")

    def portal_copy(self, ids, REQUEST=None, RESPONSE=None):
        "Wrapper around manage_copyObjects, avoids management interface"
        self.manage_copyObjects(ids, REQUEST, RESPONSE)
        return self.portal_contents(self, REQUEST,
                                    portal_status_message="Copied")

    def portal_paste(self, cb_copy_data=None, REQUEST=None):
        "Wrapper around manage_pasteObjects, avoids management interface"
        self.manage_pasteObjects(cb_copy_data, REQUEST)
        return self.portal_contents(self, REQUEST,
                                    portal_status_message="Pasted")

    portal_renameForm = HTMLFile("dtml/portal_renameForm",globals())

    def portal_rename(self, id, new_id, REQUEST=None):
        """Rename a particular sub-object while avoiding mngmnt interface"""
        try: self._checkId(new_id)
        except: raise CopyError, MessageDialog(
                      title='Invalid Id',
                      message=sys.exc_value,
                      action ='manage_main')
        ob=self._getOb(id)
        if not ob.cb_isMoveable():
            raise CopyError, eNotSupported % id            
        self._verifyObjectPaste(ob, REQUEST)
        try:    ob._notifyOfCopyTo(self, op=1)
        except: raise CopyError, MessageDialog(
                      title='Rename Error',
                      message=sys.exc_value,
                      action ='manage_main')
        self._delObject(id)
        if hasattr(ob, 'aq_base'):
            ob=ob.aq_base
        ob._setId(new_id)
        self._setObject(new_id, ob)
        #ob=ob.__of__(self)            
        #ob._postCopy(self, op=1)
        if REQUEST is not None:
            return self.portal_contents(
                self, REQUEST, portal_status_message="Renamed")
        return None

    def portal_delete(self, ids=[], REQUEST=None):
        "Avoid management interface"
        for id in ids:
            self._delObject(id)
        return self.portal_contents(self, REQUEST,
                                    portal_status_message="Deleted")

manage_addPortalFolder = PortalFolder.manage_addPortalFolder

Globals.default__class_init__(PortalFolder)
