##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
"""Base class for objects which drive the Toolbox."""

__version__='$Revision: 1.5 $'[11:-2]

from ExtensionClass import Base

class ToolboxActionProvider(Base):
    """
    
    This class is responsible for reporting the actions an object
    would like to publish in the toolbox.

    """

    def toolbox_actions(self, url=0):
        """
        Report possible toolbox actions
            
        Return: a tuple of dictionaries
            Each dictionary should describe a toolbox action.
            The items of each dictionary are:
                'name' -- string, the name of the action to be displayed
                          in the toolbox.
                          
                'url' -- the url of the action, to be called by the toolbox
                         if the action is chosen.
                         
                'permissions' -- a sequence of permissions any of which
                                 is sufficient to perform the action.
                                 
                'category' -- string 'user' | 'object' | 'global' which
                              describes the category of the action, this
                              is used by the toolbox to display the action.
        
        Permission: 'View management screens'
        """

        return ()

    def get_toolbox_actions(self, klass=0):
        """
        
        Walk my class hierarchy and return a concatenation of the
        results of the toolbox_actions methods.

        This method should only return one item per unique 'url'
        value.  The value from the class closest to the object's
        actual class should be used.

        """
        if klass == 0: klass = self.__class__
        #filtered_actions = {}
        filtered_actions = []
        urls_seen = {}
        klasses={}
        self._get_toolbox_actions(
            klass, self.absolute_url(),
            filtered_actions, urls_seen, urls_seen.has_key,
            klasses, klasses.has_key,
            )
        return tuple(filtered_actions)
    
    def _get_toolbox_actions( self, klass, absolute_url, filtered_actions
                            , urls_seen, have_url, klasses, have_klass):
        """
        
        Walk my class hierarchy and return a concatenation of the
        results of the toolbox_actions methods.

        This method should only return one item per unique 'url'
        value.  The value from the class closest to the object's
        actual class should be used.

        """
        if klass.__dict__.has_key('toolbox_actions'):
            for action in klass.toolbox_actions(self, absolute_url):
                url=action['url']
                if not have_url(url):
                  filtered_actions.append( action )
                  urls_seen[url] = 1

        klasses[klass]=1
        for base in klass.__bases__:
            if have_klass(base): continue
            self._get_toolbox_actions(
                base, absolute_url, filtered_actions, urls_seen,
                have_url, klasses, have_klass)

        


