##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
"""Portal services base objects"""

__version__='$Revision: 1.30 $'[11:-2]

import MemberFolder, PortalObject, PortalContent, PortalFolder
import OFS.PropertySheets

__ac_permissions__=(
    ('Request review', ()),
    ('Review portal content', ()),
    ('Contribute portal content', ()),
    ('Reply to item', ()),
    ('Manage portal', ()),
    ('Add portal content', ()),
    )

#
#  Construct the ZClass objects now, so that they will be available
#  immediately when the module is imported, so that through-the-web
#  products can load properly, even if they are loaded before this
#  module's initialize() is called.
#

def make_ZClass_for_base( base_class, module ):
    """
        Construct a mindless ZClass, given a Python base class, at
        module scope in 'module'.
    """
    d={}
    zclass_name = '_ZClass_for_%s' % base_class.__name__
    exec 'class %s: pass' % zclass_name in d
    Z = d[ zclass_name ]
    Z.propertysheets = OFS.PropertySheets.PropertySheets()
    Z._zclass_ = base_class
    Z.manage_options = ()
    Z.__module__ = module.__name__
    setattr( module, zclass_name, Z )

import sys
this_module = sys.modules[ __name__ ]

make_ZClass_for_base( PortalObject.PortalObjectBase, this_module )
make_ZClass_for_base( MemberFolder.MemberFolderBase, this_module )
make_ZClass_for_base( MemberFolder.MemberBase, this_module )
make_ZClass_for_base( PortalFolder.PortalFolder, this_module )
make_ZClass_for_base( PortalContent.PortalContent, this_module )

def initialize(context):
    try:
        context.registerZClass(_ZClass_for_MemberFolderBase,
                                  'PTK Member Folder Base')
        context.registerZClass(_ZClass_for_MemberBase,
                                  'PTK Member Base')
        context.registerZClass(_ZClass_for_PortalObjectBase,
                                  'PTK Portal Base')
        context.registerZClass(_ZClass_for_PortalFolder, 'PortalFolder')
        
        # Portal Content objects
        context.registerZClass(_ZClass_for_PortalContent,
                                  'Portal Content Base')

        context.registerHelpTitle( 'PTK Base Help' )
        context.registerHelp(directory='interfaces')
        
    except:
        print 'Something blew a gasket while initializing PTKBase:'
        import traceback
        traceback.print_exc()
