class ContentManager:
    """
    This interface defines the API used to provide the PTK's Content
    Manager's interface.

    Most of this interface is implemented by PortalFolder.
    Unfortunately, the 'undo' portions of this interface are presently
    implemented by PortalObject.  This shall soon change.  Since
    PortalObject is-a PortalFolder, and since all PortalFolders exist
    within a PortalObject and so aquire PortalObject's undo methods,
    this should not have any impact on how the interface is used.

    The implementations of this interface are in poor shape.  Some
    simply wrap the Zope Content Management interface's methods, some
    have had that interface's implementation pasted into them.  The
    reasons for this difference are purely hysterical, and soon all
    will be implemented the same way, probably wrapping.

    I wonder if portal_contents should be renamed to something which
    would also be appropreate as the PortalContents object's edit
    method.  This would make the PTK Content Management interface
    somewhat simpler to implement, and improve the PTK's consistentcy.

    Most methods always return HTML.  Should check to see if REQUEST
    is not None.

    Otherwise, this interface seems stable.

    Uh oh, this interface should have permissions assigned to it's
    methods.  Uh oh again, I just found their permissions, they're
    still on PortalObject!
    """

    # Implemented by PortalFolder
    
    def portal_contents(self, REQUEST):
        """
        portal_contents is a DTML object.

        This object is used to display the contents of a
        ContentManager in a friendly and non-threatening fashion.  It
        should provide access to the copy, cut, paste, create and undo
        opperations.
        """

    def portal_cut(self, ids, REQUEST=None):
        """
        This is a wrapper around manage_cutObjects.  It's sole purpose
        is to avoid sending the user to the Zope Content Management
        UI.

        Returns: HTML (portal_contents)
        Permissions: None assigned
        """

    def portal_copy(self, ids, REQUEST=None, RESPONSE=None):
        """
        Wrapper around manage_copyObjects.

        Returns: HTML (portal_contents)
        Permissions: None assigned
        """

    def portal_paste(self, cb_copy_data=None, REQUEST=None):
        """
        Wrapper around manage_pasteObjects.

        Returns: HTML (portal_contents)
        Permissions: None assigned
        """

    def portal_renameForm(self, REQUEST):
        """
        DTML object.

        Simply presents an interface to portal_rename.
        """

    def portal_rename(self, id, new_id, REQUEST=None):
        """
        Should be a wrapper around manage_renameObject, instead
        re-implements manage_renameObject.

        Returns: HTML (portal_contents) if REQUEST is not None, None otherwise
        Permissions: None assigned
        """

    def portal_delete(self, ids=[], REQUEST=None):
        """
        Wrapper around manage_deleteObjects.

        Returns: HTML (portal_contents)
        Permissions: None assigned
        """

    # Implemented by PortalObject-- Move to PortalFolder!

    def portal_undoForm(self, REQEUST):
        """
        DTML object.

        This form uses portal_undoable_transactions to present an
        interface to undo transactions which the user is permitted to
        undo.
        """

    def portal_undoable_transactions(self, AUTHENTICATION_PATH=None,
                                     first_transaction=None,
                                     last_transaction=None,
                                     PrincipiaUndoBatchSize=None,
                                     REQUEST=None):
        """
        This method is identical to Zope's 'undoable_transactions',
        except that it only returns information about the transactions
        of the present user, unless that user is a manager.

        For more information, see the Zope method.
        """

    def portal_undo(self, transaction_info, REQUEST=None):
        """
        Performs undo operations listed in transaction_info.

        Returns: HTML (portal_contents) if REQUEST is not None, None
        otherwise
        Permissions: None assigned
        """

