class DublinCore:
    """
    The Dublin Core interface defines which Dublin Core elements are
    supported by the PTK, and how.

    This interface is implemented by the DublinCore mix-in class.
    """

    def Title(self):
        """
        Dublin Core element - resource name

        Returns: string
        Permissions: View
        """
        
    def Creator(self):
        """
        Dublin Core element - resource creator

        Returns the name of the first owner found.

        Returns: string
        Permission: View
        """

    def Contributor(self):
        """
        Presently an alias for Creator
        """
    
    def Date(self):
        """
        Dublin Core element - effective date

        Returns the object's effective date if it has been explicitly
        set.  Otherwise, returns object's modification date.
        
        Returns: DateTime object
        Permissions: View
        """

    def Description(self):
        """
        Dublin Core element - summary

        Returns a natural language description of this object.
        Default implementation looks for a 'description' property on
        'self'.

        Returns: string
        Permissions: View
        """

    def Subject(self):
        """
        Dublin Core element - Topical keywords

        Returns a list of strings suitable for classifying and marking
        content.  Default implementation looks for a 'subject'
        property on 'self'.  If 'subject' is a tuple or string, this
        method massages it into a list before returning it.
        
        Returns: list of strings
        Permissions: View
        """

    def Type(self):
        """
        Dublin Core element - Object type

        Returns object's 'meta_type'.

        Returns: string
        Permissions: View
        """

    def Identifier(self):
        """
        Dublin Core element - Object ID

        Returns object's 'id'.

        Returns: string
        Permissions: View
        """

