class PortalInterface:
    """

    This is the interface of the basic portal services.

    Parts of this interface are quite raw.
    
    This interface is implemented by PortalObject.
    """

    def getAuthenticatedMember(self):
        '''
        Returns the currently authenticated member object or None.
        '''

    def getMemberById(self):
        '''
        Returns the given member.
        '''

    def addMemberFolder(self, REQUEST=None):
        """

        Add the member folder to the the portal site object.  The
        member folder must fulfill the BasicUserFolder interface.  The
        member folder it creates may or may not be called 'acl_users'.  This
        method is called durring portal initialization.  This
        implementation instantiates an instance of self.MembersClass.
        The PTK does not define a value for this (MembersClass)
        attribute.  It is expected that the application has
        initialized this to it's desired membership class.

        This will all change shortly.  It's expected that this method
        will instantiate a LoginManager object, and then populate it
        with whatever methods may be needed.  Chances are, no
        additional methods will be needed, but some potential hooks
        may be defined.  It will probably also create the initial
        UserSource and LoginMethod objects in the LoginManager, from
        choices made by the user in the Portal Wizard.

        Returns: HTML if the REQUEST argument was passed, or if the
        member folder already exists.

        Permissions: Manage portal
        """

    def addMember(self, name, password, roles, domains):
        """
        Create a new Member with the given information.  Does not set
        up any additional Member resources, such as the home folder.
        Should applications extend this method to do so?

        Returns: None
        Permissions: None assigned
        """

    def register(self, username='', full_name='', company='', email='',
                 REQUEST=None):
        """

        This method is responsible for validating a request to add a
        user, and for actually performing the add.

        This method expects to find a DTML object on self called
        'join_form'.  In case of invalid user input, it will call this
        object, passing it a string as 'error' explaining what was
        invalid, and returns the result.  This gives the user an
        opportunity to correct their input.  The form should be able
        to fill in it's fields with the original values from the
        REQUEST variable in this case.

        Presently, it calls 'addMember' on the 'acl_users' object to
        perform the creation.  This method will be going away when the
        switch to LoginManager is made, at which point this method
        will begin calling 'addMember' on self.

        Once it has created the user, it calls the user object's
        setMemberProperties method, passing the REQUEST variable.
        This initializes the member's propertysheets to the values
        specified in the input form.  This should perhaps be done in
        addMember.

        This method calls self.register_hook.  See 'PortalPolicy.py'
        for information.

        This method calls self.registered_notify, passing the REQUEST
        and the password the portal has generated for the user.
        registered_notify is expected to email or otherwise notifty
        the user of their password.  Perhaps this should be done by
        register_hook.

        Finally, if successful, this method returns the result of
        calling self's 'registered' method.  register expects
        'registered' to be a DTML method.  This method should tell the
        user how to discover their password.
        
        This method's signature is mostly bogus legacy.  Only usename,
        email and REQUEST are actually used.  This should be cleaned
        up.

        TODO: Quite a lot, read the above carefully.

        Returns: HTML
        Permission: None assigned.
        """

    def mail_password(self, forgotten_username, REQUEST):
        """

        This method is called by the user (via filling in a form) in
        the inevitable case that they have forgotten their password.
        After validation, it performs this action by calling a method
        on self called 'mail_password_template'.  The portal expects
        this to be a DTML document, and returns the result of calling
        it.

        'forgotten_username' - the username of the forgetful user.

        Returns: HTML
        Permissions: None assigned.
        """

    def changePassword(self, user, password, confirm, domains, REQUEST,
                       redirect=None):
        """

        This method is essentially a wrapper around
        _changeUser().  It verifies that password == confirm
        and that the user is the owner of the password, then calls
        _changeUser.

        After changing the password, this method does what is
        necessary to update the client's credentials.

        If 'redirect' is passed in, this method redirects the client
        to the address it specifies.  Otherwise, this method expects
        to find on self a DTML object called 'personalize_form', which
        it returns the result of calling.  It passes in a status
        message via the 'portal_status_message' argument, which should
        be interpreted by the standard_html_header.

        Returns: HTML
        Permissions: None assigned.
        """

    def getCatalog(self):
        """

        This method returns the Portal's ZCatalog object.  The default
        implementation expects to find the catalog as 'SiteIndex'.

        Returns: ZCatalog object
        Permissions: None assigned
        """

    def getPortal(self):
        """

        This method returns the actual Portal object.

        Returns: PortalObject (or subclass)
        Permissions: None assigned
        """

    def getToolbox(self):
        """

        Return's an object which implements the Toolbox interface.
        The default implementation instantiates and returns a Toolbox
        object.

        Returns: Object implementing Toolbox interface
        Permissions: None assigned
        """

    def portal_url(self):
        """

        Return the URL of the portal object, in the client's reckoning.

        Returns: String containing portal's URL
        Permissions: None assigned
        """
