class ToolboxAware:
    """
    Interface for content which is manageable by the Toolbox.

    CHANGE: Object now automatically publish ancesteral
    toolbox_actions as well as their classes' own.
    """
    
    def toolbox_actions(self):
        """
        Report this class's potential actions
            
        Return: a sequence of dictionaries
            Each dictionary should describe a toolbox action.
            The items of each dictionary are:
                'name' -- string, the name of the action to be displayed
                          in the toolbox.
                          
                'url' -- the url of the action, to be called by the toolbox
                         if the action is chosen.
                         
                'permissions' -- a sequence of permissions any of which
                                 is sufficient to perform the action.
                                 
                'category' -- string 'user' | 'object' | 'global' which
                              describes the category of the action, this
                              is used by the toolbox to display the action.
        
        """

    def get_toolbox_actions(self):
        """
        Report this _object's_ potential actions.  This is done by
        walking the class hierarchy and collecting the results from
        any toolbox_actions methods found.  If two actions with the
        same 'url' are found, the one least removed from the object's
        actual class is used.
        
        This method is called by Toolbox.
        
        Returns the same as toolbox_actions.
        """
