import urllib, string

import Globals
from ExtensionClass import Base
from Globals import HTMLFile
from DateTime import DateTime

class Discussable(Base):
    """
    This is the mixin for PortalContent items which are discussable
    """

    __ac_permissions__ = (
        ('Reply to item',
         ('replyForm', 'replyPreview', 'createReply'),
         ('Member',)
         ),
        ('View', ('getReplyResults', 'getReplies'))
        )

    threadView = HTMLFile('dtml/threadView',globals())
    replyForm = HTMLFile('dtml/replyForm', globals())
    replyPreview = HTMLFile('dtml/replyPreview', globals())
    
    def createReply(self, title, text, REQUEST, RESPONSE):
        """
        Create a reply in the proper place
        """

        location, id = self.getReplyLocationAndID(REQUEST)
        location.addDiscussionItem(id, title, title, 'structured-text',
                                   text, self)

        RESPONSE.redirect(self.absolute_url() + '/threadView')

    def getReplyLocationAndID(self, REQUEST):
        # It is not yet clear to me what the correct location for this hook is

        # Find the folder designated for replies, creating if missing
        home = self.getPortal().getAuthenticatedMember().getHomeFolder()
        if not hasattr(home, 'Correspondence'):
            home.manage_addPortalFolder('Correspondence')
        location = home.Correspondence

        # Find an unused id in location
        id = int(DateTime().timeTime())
        while hasattr(location, `id`):
            id = id + 1

        return location, `id`

    def getReplyResults(self):
        """
        Return the ZCatalog results that represent this object's replies.

        Often, the actual objects are not needed.  This is less expensive
        than fetching the objects.
        """
        catalog = self.getPortal().getCatalog()
        return catalog.searchResults(in_reply_to=
                                      urllib.unquote('/'+self.absolute_url(1)))

    def getReplies(self):
        """
        Return a sequence of the DiscussionResponse objects which are
        associated with this Discussable
        """
        catalog = self.getPortal().getCatalog()
        results = self.getReplyResults()
        rids    = map(lambda x: x.data_record_id_, results)
        objects = map(catalog.getobject, rids)
        return objects

    def quotedContents(self):
        """
        Return this object's contents in a form suitable for inclusion
        as a quote in a response.
        """

        return ""

    def allowReplies(self):
        return 1

    def toolbox_actions(self, url=0):
        # Return actions for reply and show replies

        if url==0: url=self.absolute_url()
        
        reply_action = (
            {'name': 'Reply',
             'url': url + '/replyForm',
             'permissions': ['Reply to item'],
             'category': 'object'
             },
            )

        show_replies_action = (
            {'name': 'View thread',
             'url': url + '/threadView',
             'permissions': ['View'],
             'category': 'object'
             },
            )
        
        actions=show_replies_action
        if self.allowReplies():
            actions = actions + reply_action
        return actions
                       

class DiscussionResponse(Base):
    """
    This is the mixin for PortalContent items which may be responses
    """

    in_reply_to = ''

    # XXX I could probably remove a case from the resolution code
    
    def inReplyTo(self, REQUEST=None):
        """
        Return the Discussable object which this item is associated with
        """
        # Stolen from ZCatalog.resolve_url
        
        if REQUEST is None:
            REQUEST = self.REQUEST

        script = REQUEST.script
        path = self.in_reply_to
        if string.find(path, script) != 0:
            path='%s/%s' % (script, path)
        return REQUEST.resolve_url(path)

    def setReplyTo(self, reply_to):
        """
        Make this object a response to the passed object
        """

        if type(reply_to) == type(''):
            REQUEST = self.REQUEST
            script = REQUEST.script
            if string.find(reply_to, script) != 0:
                reply_to='%s/%s' % (script,reply_to)
            reply_to = REQUEST.resolve_url(reply_to)

        if not hasattr(reply_to, 'allowReplies') or \
           not reply_to.allowReplies():
            raise ValueError, "reply_to does not accept replies"
            
        self.in_reply_to = urllib.unquote('/' + reply_to.absolute_url(1))
        if hasattr(self, 'getPortal'):
            portal = self.getPortal()
            if portal is not None:
                # getPortal won't be available when this method is called while
                # the object is being constructed
                portal.notify_content(
                    self, self.getPortal().getAuthenticatedMember())

    def parentsInThread(self, size=0):
        """
        Return the list of object which are this object's parents, from the
        point of view of the threaded discussion.  Parents are ordered
        oldest to newest.

        If 'size' is not zero, only the closest 'size' parents will be
        returned.
        """

        parents = []
        parent = self.inReplyTo()

        while parent and (not size or len(parents) < size):
            if parent in parents:
                # Paranoia: circular thread
                print self.absolute_url(), "is in a circular thread"
                break
            parents.append(parent)
            parent = hasattr(parent, 'inReplyTo') and parent.inReplyTo()
        parents.reverse()
        return parents

Globals.default__class_init__(Discussable)
