"""
This module implements a portal-managed File class.  It is based on
Zope's built-in File object, but modifies the behaviour slightly to
make it more Portal-friendly.
"""

from Globals import HTMLFile, HTML
from Products.PTKBase.PortalContent import PortalContent
from Discussions import Discussable
from Acquisition import Implicit
import Globals

from OFS import Image

def addFile(self, id, file, title='', description='', precondition='',
            content_type='', RESPONSE=None):
    """
    Add a File
    """

    # cookId sets the id and title if they are not explicity specified
    id, title = Image.cookId(id, title, file)

    self=self.this()  # Why?

    # Instantiate the object and set it's description.
    # The description is not set by the constructor because I didn't
    # want to extend File's constructor.  Perhaps I should.
    fobj = File(id, title, '', content_type, precondition)
    fobj.description = description
    
    # Add the File instance to self
    self._setObject(id, fobj)

    # 'Upload' the file.  This is done now rather than in the
    # constructor because it's faster.  Why is it faster?
    self._getOb(id).manage_upload(file)

    if RESPONSE is not None:
        RESPONSE.redirect(self.absolute_url()+'/portal_contents')
    
class File(Image.File, PortalContent, Discussable):
    """
    A Portal-managed File
    """

    # The order of base classes is very significant in this case.
    # Image.File does not store it's id in it's 'id' attribute.
    # Rather, it has an 'id' method which returns the contents of the
    # instnace's __name__ attribute.  Inheriting in the other order
    # obscures this method, resulting in much pulling of hair and
    # gnashing of teeth and fraying of nerves.  Don't do it.
    #
    # Really.
    
    meta_type='Portal File'

    __ac_permissions__ = (
        ('View', ('download',)),
        ('Change Portal Files', ('edit',), ('Owner',)),
        ('View management screens',('editForm',)),
        )
    
    def SearchableText(self):
        """
        SeachableText is used for full text seraches of a portal.  It
        should return a concatanation of all useful text.
        """
        return "%s %s" % (self.title, self.description)

    def toolbox_actions(self, url=0):
        if url==0: url=self.absolute_url()
        return (
            {'name':'View statistics',
             'url':  url + '/index_html',
             'permissions':('View',),
             'category':'object',
             },  
            {'name':'Download',
             'url':  url + '/download',
             'permissions':('View',),
             'category':'object',
             },  
            {'name':'Edit',
             'url':  self.absolute_url() + '/editForm',
             'permissions':('Change Portal Files',),
             'category':'object',
             },
            )


    def manage_afterAdd(self, item, container):
        """Both of my parents have an afterAdd method"""
        Image.File.manage_afterAdd(self, item, container)
        PortalContent.manage_afterAdd(self, item, container)

    def manage_beforeDelete(self, item, container):
        """Both of my parents have a beforeDelete method"""
        PortalContent.manage_beforeDelete(self, item, container)
        Image.File.manage_beforeDelete(self, item, container)
        

    def edit(self, title, description, content_type,
             precondition='', file='', REQUEST=None):
        """
        Perform changes for user
        """
        self.title = title
        self.description = description
        self.content_type = content_type
        if precondition: self.precondition = precondition
        elif self.precondition: del self.precondition

        if file:
            self.manage_upload(file)

        # Tell the portal I changed
        self.getPortal().notify_content(
            self, self.getPortal().getAuthenticatedMember())
        
        if REQUEST is not None:
            return self.editForm(self, REQUEST,
                                 portal_status_message='File changed.')

    def download(self, REQUEST, RESPONSE):
        """
        Download this item.  Calls OFS.Image.File.index_html to perform the
        actual transfer after first setting Content-Disposition to suggest
        a filename.
        """

        RESPONSE.setHeader('Content-Disposition',
                           'attachment; filename="%s"' % self.id())
        return Image.File.index_html(self, REQUEST, RESPONSE)

    editForm = HTMLFile('dtml/fileEdit', globals())
    index_html = HTMLFile('dtml/fileView', globals())


Globals.default__class_init__( File )

# PortalContent classes need to be registered.  This installs the
# class's factory method and icon.

from Products.PTKBase.register import registerPortalContent

registerPortalContent( File
                     , constructors=( addFile, )
                     , action='Wizards/File'
                     , icon=Globals.ImageFile( "file.gif", globals() )
                     )
