"""
This module implements a portal-managed Image class.  It is based on
Zope's built-in Image object.
"""

from Globals import HTMLFile, HTML
from Products.PTKBase.PortalContent import PortalContent
from Discussions import Discussable
from Acquisition import Implicit
import Globals

import OFS.Image

def addImage(self, id, file, title='', description='', precondition='',
            content_type='', RESPONSE=None):
    """
    Add an Image
    """

    # cookId sets the id and title if they are not explicity specified
    id, title = OFS.Image.cookId(id, title, file)

    self=self.this()  # Why?

    # Instantiate the object and set its description.
    # The description is not set by the constructor because I didn't
    # want to extend Image's constructor.  Perhaps I should.
    iobj = Image(id, title, '', content_type, precondition)
    iobj.description = description
    
    # Add the Image instance to self
    self._setObject(id, iobj)

    # 'Upload' the image.  This is done now rather than in the
    # constructor because it's faster.  Why is it faster?
    self._getOb(id).manage_upload(file)

    if RESPONSE is not None:
        RESPONSE.redirect(self.absolute_url()+'/portal_contents')
    
class Image(OFS.Image.Image, PortalContent, Discussable):
    """
    A Portal-managed Image
    """

    # The order of base classes is very significant in this case.
    # Image.Image does not store it's id in it's 'id' attribute.
    # Rather, it has an 'id' method which returns the contents of the
    # instnace's __name__ attribute.  Inheriting in the other order
    # obscures this method, resulting in much pulling of hair and
    # gnashing of teeth and fraying of nerves.  Don't do it.
    #
    # Really.
    
    meta_type='Portal Image'

    __ac_permissions__ = (
        ('Change Portal Images', ('edit',), ('Owner',)),
        ('View management screens',('editForm',)),
        )
    
    def SearchableText(self):
        """
        SeachableText is used for full text seraches of a portal.  It
        should return a concatanation of all useful text.
        """
        return "%s %s" % (self.title, self.description)

    def toolbox_actions(self, url=0):
        if url==0: url=self.absolute_url()
        return (
            {'name':'Edit',
             'url':  url + '/editForm',
             'permissions':('Change Portal Images',),
             'category':'object',
             },
            )


    def manage_afterAdd(self, item, container):
        """Both of my parents have an afterAdd method"""
        OFS.Image.Image.manage_afterAdd(self, item, container)
        PortalContent.manage_afterAdd(self, item, container)

    def manage_beforeDelete(self, item, container):
        """Both of my parents have a beforeDelete method"""
        PortalContent.manage_beforeDelete(self, item, container)
        OFS.Image.Image.manage_beforeDelete(self, item, container)
        

    def edit(self, title, description, content_type,
             precondition='', file='', REQUEST=None):
        """
        Perform changes for user
        """
        self.title = title
        self.description = description
        self.content_type = content_type
        if precondition: self.precondition = precondition
        elif self.precondition: del self.precondition

        if file:
            self.manage_upload(file)

        # Tell the portal I changed
        self.getPortal().notify_content(
            self, self.getPortal().getAuthenticatedMember())
        
        if REQUEST is not None:
            return self.editForm(self, REQUEST,
                                 portal_status_message='Image changed.')

    editForm = HTMLFile('dtml/imageEdit', globals())

    view = HTMLFile('dtml/imageView', globals())
    
    def index_html(self, REQUEST, RESPONSE):
        """
        Display the image, with our without standard_html_[header|footer],
        as appropreate.
        """
        if REQUEST['PATH_INFO'][-10:] == 'index_html':
            return self.view(self, REQUEST)
        return OFS.Image.Image.index_html(self, REQUEST, RESPONSE)

Globals.default__class_init__( Image )


from Products.PTKBase.register import registerPortalContent

registerPortalContent( Image
                     , constructors=( addImage, )
                     , action='Wizards/Image'
                     , icon=Globals.ImageFile( "image.gif", globals() )
                     )
