from Products.PTKBase.PortalCatalog import PortalCatalog
from Products.PTKBase.PortalFolder import *
import Document

default_member_content = """\
Default page for %s

  This is the default document created for you when 
  you joined this community.

  To change the content just select "Edit"
  in the Tool Box on the left.
"""

members_roster = """\
<dtml-var standard_html_header>

<p>This roster page doesn't batch, it's just a placeholder so far.</p>

<dtml-in "getRoster(REQUEST)">
 <a href="&dtml.url_quote-sequence-item;">&dtml-sequence-item;</a><br>
<dtml-else>
 <p><b>No listed members.</b></p>
</dtml-in>

<dtml-var standard_html_footer>
"""

class DemoPortalBase:
    ' '

    __ac_permissions__=(
        ('Manage portal',
         ('install', 'portal_reconfig', 'refresh_interface', 'apply_wizard'),
         ('Owner',)),
        )

    _using_LoginManager = 0

    def _getPUS(self):
        if self._using_LoginManager:
            # Get the PersistentUserSource object that stores our users
            for us in self.acl_users.UserSourcesGroup.objectValues():
                if us.meta_type == 'Persistent User Source':
                    # The following works around a PlugIn bug
                    return us.__of__(self.acl_users)
        else:
            return self.acl_users

    def install(self, REQUEST=None):
        """Set up initial components  --  called by ZClass constructor"""

        if REQUEST is None:
            REQUEST = self.REQUEST

        # Add Members folder
        manage_addPortalFolder(self, 'Members')
        self.Members.manage_addDTMLMethod('index_html', 'Members',
                                          members_roster)

        # Determine membership storage and create acl_users
        #  Because SQLMemberFolder requires nonstandard arguments, I cannot
        #  use PTKBase.PortalBase.addMemberFolder
        db = REQUEST['database']
        if db == 'ZODB':
            self.MembersClass = self.ZODBMemberFolder
            acl_users = self.MembersClass()
            self.this()._setObject('acl_users', acl_users)
        elif db == 'LoginManager':
            self._using_LoginManager = 1
            from Products.LoginManager.LoginManager import \
                 LoginManager, manage_addLoginManager
            from Products.Membership.PersistentUserSource \
                 import PersistentUserSource
            self.MembersClass = LoginManager
            manage_addLoginManager(self,
                                   'Persistent User Source',
                                   ['Basic Cookie Login', 'Basic Auth Login'],
                                   0, 0, 0)
            # self.acl_users was magically created by the above call
            self._getPUS().manage_setStorage('MembershipZ/LoginMember')
        else:
            self.MembersClass = self.SQLMemberFolder
            acl_users = self.MembersClass(db)
            self.this()._setObject('acl_users', acl_users)

        # I'm not sure why this is done via this() but this is what
        # addMemberFolder does, so I do as well.
        self.this().__allow_groups__=self.acl_users

        # Install PortalCatalog
        id = 'SiteIndex'
        catalog = PortalCatalog(id, '')
        self._setObject(id, catalog)
        catalog = self.SiteIndex
        catalog.initialize()

        # Create roles
        self._addRole('Member')
        self._addRole('Reviewer')
        self._addRole('Contributor')

        # Give permissions out
        self.manage_role('Owner', [
            'View management screens',
            'Add portal content',
            'Add Folders',
            'Delete objects',
            'FTP access',
            'Manage properties',
            'Undo changes',
            'Request review'
            ])
        self.manage_role('Reviewer', [
            'Review portal content',
            ])
        self.manage_role('Contributor', [
            'Contribute portal content',
            ])
        self.manage_role('Member', [
            'Reply to item',
            ])

    def portal_reconfig(self, REQUEST):
        "Accept changes from portal_config"
        self.propertysheets.PortalProperties.manage_changeProperties(REQUEST)
        self.MailHost.smtpHost = REQUEST['smtp_server']
        return self.portal_config(self, REQUEST,
                                  portal_status_message="Settings changed")

    def before_review(self, changed_object, action, review_state, comments,
                      REQUEST):
        # Someone has just tried to change the reviewed_state of the object.
        # This is an opportunity to modify the change before it takes place,
        # or reject it entierly.
        #
        # 'changed_object' is a slight misnomner-- it hasn't changed _yet_.
        if REQUEST.AUTHENTICATED_USER.has_permission(
            'Contribute portal content', changed_object) and review_state=='pending':
            # This user is a contributer-- skip reviewing
            review_state = 'published'
            action = 'Item published'
        return action, review_state, comments

    def personalize(self, REQUEST):
        "Called by personalize_form to commit changes"
        user = REQUEST.AUTHENTICATED_USER
        user.setMemberProperties(REQUEST)

        return self.personalize_form(self, REQUEST,
                                     portal_status_message="Member changed.")

    def getRoster(self, REQUEST):
        """
        Return a list of the usernames of those users who have made themselves
        "listed".  If Manager, return a list of all usernames.
        """
        # Consider changing this to check for a permission rather than a role
        if REQUEST.AUTHENTICATED_USER.has_role('Manager'):
            return self._getPUS().getUserNames()
        names = []
        for user in self._getPUS().getUsers():
            if user.listed:
                names.append(user.getUserName())
        return names

    def addMember(self, name, password, roles, domains):
        """
        This extends the PTK's addMember to support the proto-LoginManager.
        The final LM will have a simpler interface than this.
        """
        if self.acl_users.meta_type == 'Login Manager':
            # The acl_users folder is a LoginManager.  Search for a UserSource
            # with the needed support.
            for source in self.acl_users.UserSourcesGroup.objectValues():
                if hasattr(source, 'addUser'):
                    source.__of__(self).addUser(name, password, roles, domains)
                    return
            # XXX Bogus exception
            raise "Can't add Member", "No supported UserSources"
        else:
            self.acl_users._addUser(name, password, password, roles, domains)



    # Portal Wizard methods

    def refresh_interface(self, interface):
        """
        Replace the present interface with the one specified.
        This can be used to update an unchanged interface when the ZClasses
        have been upgraded.
        """

        # Copy from the specified Interface Folder to the Portal instance.
        # This will OVERWRITE existing objects.
        source = self.Control_Panel.Products.DemoPortal.Interfaces[interface]
        for ob in source.objectValues():
            ob = ob._getCopy(source)
            try:   id = ob.id()
            except:id = ob.id
            if hasattr(self.aq_base, id):
                self._delObject(id)
            self._setObject(id, ob)

        # Wizards need to be handled way better XXX
        self.Wizards.manage_permission('View',['Member'],1) # let Members see Wizards

    def register_hook(self, REQUEST):
        """
        Called when a new user is registered
        """
        name=REQUEST['username']

        # Create Member's home folder
        manage_addPortalFolder(self.Members, name)
        f=getattr(self.Members,name)

        # set permissions on home folder
        f.manage_permission('View', ['Owner','Manager','Reviewer'])
        f.manage_permission('View management screens', ['Owner', 'Manager'])
        f.manage_permission('Delete objects', ['Owner', 'Manager'])

    # Create Member's home page
        Document.addDocument(
            f,
            'index_html',
            name+"'s Home",
            name+"'s front page",
            "structured-text",
            default_member_content % (name)
            )

        # Grant ownership to Member
        f.manage_setLocalRoles(name, ['Owner'])

    def apply_wizard(self, REQUEST):
        "Apply changes from filling in Portal Wizard"

        if not REQUEST.AUTHENTICATED_USER.has_role('Manager', self):
            raise 'Unauthorized'

        interface = REQUEST['interface']
        username = getattr(REQUEST, 'username', None)
        password = getattr(REQUEST, 'password', None)
        confirm = getattr(REQUEST, 'confirm', None)
        smtp_server = REQUEST['smtp_server']

        # Save info from Request in properties.
        self.propertysheets.PortalProperties.manage_changeProperties(REQUEST)

        # Create MailHost for mailing items from the portal
        self.manage_addMailHost('MailHost', smtp_host=smtp_server)

        if username:
            # Create manager's account.
            if password != confirm:
                raise "Password mismatch"
            self.addMember(username, password,
                           ['Member', 'Manager', 'Reviewer'], [])
            self.acl_users.getUser(username).setMemberProperties(REQUEST)

            # Create manager's home folder
            self.register_hook(REQUEST)

        # Copy the interface
        self.refresh_interface(interface)

    def manage_migrate_content(self, REQUEST):
        '''Converts instances of Products.PTKBase.<content> to
        instances of Products.PTKDemo.<content>.'''
        import Products.PTKBase.Document
        import Products.PTKBase.File
        import Products.PTKBase.Image
        import Products.PTKBase.Link
        import Products.PTKBase.NewsItem
        import NewsItem, Link, Document, File, Image

        migrations = {
            Products.PTKBase.Document.Document : Document.Document,
            Products.PTKBase.File.File         : File.File,
            Products.PTKBase.Image.Image       : Image.Image,
            Products.PTKBase.Link.Link         : Link.Link,
            Products.PTKBase.NewsItem.NewsItem : NewsItem.NewsItem,
            }

        visited = []
        migrated = []
        self.__migrate_branches(migrations, self, migrated, visited)
        from string import join
        return 'Converted:\n%s\n\nDone.' % join(migrated, '\n')

    def __migrate_branches(self, migrations, branch, migrated, visited):
        base = getattr(branch, 'aq_base', branch)
        if base in visited:
            # Don't visit again!
            return
        visited.append(base)

        try: changed = branch._p_changed
        except: changed = 1
        for id in branch.objectIds():
            obj = branch._getOb(id)
            obase = getattr(obj, 'aq_base', obj)
            klass = obase.__class__
            if migrations.has_key(klass):
                # Replace this object.
                changed = 1
                newob = migrations[klass]()
                newob.id = obase.id   # This line activates obase.
                newob.__dict__.update(obase.__dict__)
                setattr(branch, id, newob)
                migrated.append(obj.absolute_url())
            elif hasattr(obase, 'objectIds'):
                # Enter a sub-branch.
                self.__migrate_branches(migrations, obj, migrated, visited)
            else:
                # Unload this object if it has not been changed.
                try:
                    if obj._p_changed is None:
                        obj._p_deactivate()
                except: pass
        if changed is None:
            # Unload this branch.
            object._p_deactivate()

        del visited[-1]
