from Products.PTKBase.MemberFolder import MemberBase, MemberFolderBase
from Globals import HTMLFile
from Products.ZSQLMethods.SQL import SQL

import string

class DemoSQLMemberBase:
    """Demonstration SQL-based Member mix-in class"""

    def __init__(self,name,password,roles,domains):
        self.name   =name
        self.__     =password
        self.roles  =filter(None,map(string.strip,string.split(roles, ',')))
        self.domains=filter(None,map(string.strip,string.split(domains, ',')))

    def setMemberProperties(self, REQUEST):
        for ps in self.propertysheets:
            ps.manage_changeProperties(REQUEST)
            # Get the list of properties of the present propertysheet which
            # are found in REQUEST
            ids = filter(lambda x, k=REQUEST.keys(): x in k, ps.propertyIds())
            for propid in ids:
                # Check if property exists, update if so, insert otherwise.
                # There is probably a much better way to do this.  :-/
                if self.getSqlProperty(username=self.name,
                                       propertysheet=ps.id,
                                       property=propid):
                    self.updateSqlProperty(
                        username=self.name,
                        propertysheet=ps.id,
                        property=propid,
                        value=REQUEST[propid]
                        )
                else:
                    self.insertSqlProperty(
                        username=self.name,
                        propertysheet=ps.id,
                        property=propid,
                        value=REQUEST[propid]
                        )

class DemoSQLMemberFolderBase:
    """Demonstration SQL-based Member Folder mix-in class"""

    #_editUser = HTMLFile('editUser', globals())
    
    def __init__(self, conn_id):
        self.conn_id = conn_id

        # User methods
        self.sqlListQuery=SQL('sqlListQuery', '', conn_id, '', _sqlListQuery)
        self.sqlUserQuery=SQL('sqlUserQuery', '', conn_id, 'username', _sqlUserQuery)
        self.sqlEditQuery=SQL('sqlEditQuery', '', conn_id, 'username:required password:required domains roles', _sqlEditQuery)
        self.sqlAddQuery=SQL('sqlAddQuery', '', conn_id, 'username password domains roles', _sqlAddQuery)
        self.sqlDelQuery=SQL('sqlDelQuery', '', conn_id, 'username', _sqlDelQuery)

        # Propertysheet methods
        self.getSqlPropertysheet=SQL('getSqlPropertysheet', '', conn_id, 'username:required propertysheet:required', _getSqlPropertysheet)
        self.getSqlProperty=SQL('getSqlProperty', '', conn_id, 'username:required propertysheet:required property:required', _getSqlProperty)
        self.insertSqlProperty=SQL('insertSqlProperty', '', conn_id, 'username:required propertysheet:required property:required value', _insertSqlProperty)
        self.updateSqlProperty=SQL('updateSqlProperty', '', conn_id, 'username:required propertysheet:required property:required value', _updateSqlProperty)
                                    

    def getUserNames(self):
        """Return a list of usernames"""
        data = []
        items=self.sqlListQuery()
        for ob in items:
            data.append(sqlattr(ob, 'username'))
        data.sort()
        return data

    def getUsers(self):
        """Return a list of user objects"""
        data = []
        items = self.sqlListQuery()
        for ob in items:
            user = self.MemberClass(sqlattr(ob, 'username'),
                                    sqlattr(ob, 'password'),
                                    sqlattr(ob, 'roles'),
                                    sqlattr(ob, 'domains'))
            # Load propertysheets from SQL database
            for ps in user.propertysheets:
                for property in self.getSqlPropertysheet(username=user.name,
                                                         propertysheet=ps.id):
                    ps._updateProperty(property.property, property.value)
            data.append(user)
        return data

    def getUser(self, name):
        """Return the named user object or None"""
        ob = self.sqlUserQuery(username=name)
        if not ob: return None
        ob=ob[0]
        user = self.MemberClass(sqlattr(ob, 'username'),
                                sqlattr(ob, 'password'),
                                sqlattr(ob, 'roles'),
                                sqlattr(ob, 'domains'))

        # Load propertysheets from SQL database
        for ps in user.propertysheets:
            for property in self.getSqlPropertysheet(username=user.name,
                                                     propertysheet=ps.id):
                ps._updateProperty(property.property, property.value)

        return user.__of__(self)

    def _doAddUser(self, name, password, roles, domains):
        """Create a new user"""
        self.sqlAddQuery(username=name,
                         password=password,
                         roles=string.join(roles, ','),
                         domains=string.join(domains, ',')
                         )
                       
    def _doChangeUser(self, name, password, roles, domains):
        self.sqlEditQuery(username=name,
                          password=password,
                          roles=string.join(roles, ','),
                          domains=string.join(domains, ',')
                          )

    def _doDelUsers(self, names):
        for name in names:
            self.sqlDelQuery(username=name)


# Stolen directly from UserDB

# A getattr-like helper that hopefully can compensate for the
# various oddities in data returned by different stupidbases.

import Missing
mt=type(Missing.Value)

def typeconv(val):
    if type(val)==mt:
        return ''
    return val

def sqlattr(ob, attr):
    name=attr
    if hasattr(ob, attr):
        return typeconv(getattr(ob, attr))
    attr=upper(attr)
    if hasattr(ob, attr):
        return typeconv(getattr(ob, attr))
    attr=lower(attr)
    if hasattr(ob, attr):
        return typeconv(getattr(ob, attr))
    raise NameError, name

# SQL Methods used by SQL Membership folder

_sqlListQuery="""<!--#comment-->

This query retrieves rows representing all users

schema: username  password  domains  roles

<!--#/comment-->
SELECT * FROM users
"""

_sqlUserQuery="""<!--#comment-->

This query retrieves a row representing a given user

schema: username  password  domains  roles

<!--#/comment-->
SELECT * FROM users
<!--#sqlgroup required where-->
<!--#sqltest username column="username" type="string"-->
<!--#/sqlgroup-->
"""

_sqlAddQuery="""<!--#comment-->

This query adds a new user

<!--#/comment-->
INSERT INTO users VALUES (
<!--#sqlvar username type="string"-->,
<!--#sqlvar password type="string"-->,
<!--#sqlvar domains type="string"-->,
<!--#sqlvar roles type="string"-->
)
"""

_sqlEditQuery="""<!--#comment-->

This query updates a users data

<!--#/comment-->
UPDATE users SET 
password=<!--#sqlvar password type="string"-->,
domains=<!--#sqlvar domains type="string"-->,
roles=<!--#sqlvar roles type="string"-->
<!--#sqlgroup required where-->
<!--#sqltest username column="username" type="string"-->
<!--#/sqlgroup-->
"""

_sqlDelQuery="""<!--#comment-->

This query removes a user

<!--#/comment-->
DELETE from users
<!--#sqlgroup required where-->
<!--#sqltest username column="username" type="string"-->
<!--#/sqlgroup-->
"""

# The following SQL Methods support SQL-stored Propertysheets.

_getSqlPropertysheet = """
SELECT property, value FROM propertysheets
<dtml-sqlgroup required where>
 <dtml-sqltest username type="string">
<dtml-and>
 <dtml-sqltest propertysheet type="string">
</dtml-sqlgroup>
"""

_getSqlProperty = """
SELECT value FROM propertysheets
<dtml-sqlgroup required where>
 <dtml-sqltest username type="string">
<dtml-and>
 <dtml-sqltest propertysheet type="string">
<dtml-and>
 <dtml-sqltest property type="string">
</dtml-sqlgroup>
"""
 
_insertSqlProperty = """
INSERT INTO propertysheets VALUES (
 <dtml-sqlvar username type="string">,
 <dtml-sqlvar propertysheet type="string">,
 <dtml-sqlvar property type="string">,
 <dtml-sqlvar value type="string">
)"""

_updateSqlProperty = """
UPDATE propertysheets SET
 value=<dtml-sqlvar value type="string">
<dtml-sqlgroup required where>
 <dtml-sqltest username type="string">
<dtml-and>
 <dtml-sqltest propertysheet type="string">
<dtml-and>
 <dtml-sqltest property type="string">
</dtml-sqlgroup>
"""
