##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
"""Sample portal base classes"""
__version__='$Revision: 1.5 $'[11:-2]
 
import Portal, ZODBMember, SQLMember, OFS.PropertySheets
import Document, Link, NewsItem, File, Image
import Discussions, DiscussionItem
from Products import PTKBase


#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#   N.B.:  The following symbol controls whether we "inject" the
#          content types which formerly lived in PTKBase back into
#          it.  While it is initially true (to allow existing portal
#          content to load), in a future release it will be set to
#          false;  the behavior it governs will eventually be removed
#          altogether.  YOU HAVE BEEN WARNED!!!
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
SUPPLY_DEPRECATED_PTK_BASE_ALIASES = 0

#
#  Construct the ZClass objects now, so that:
#   * they will be available immediately when the module is imported
#   * therefore, through-the-web products can load properly, even if
#     they are loaded before this module's initialize() is called.
#
import sys
this_module = sys.modules[ __name__ ]

def make_and_alias_ZClass_for_base( base_class ):
    # No docstring, coz not publishable :(
    PTKBase.make_ZClass_for_base( base_class, this_module )

    if SUPPLY_DEPRECATED_PTK_BASE_ALIASES:
        # then we also inject the ZClass into PTKBase.
        class_name = base_class.__name__
        zclass_name = '_ZClass_for_%s' % class_name
        PTKBase.__dict__[ zclass_name ] = this_module.__dict__[ zclass_name ]

PTKBase.make_ZClass_for_base( Portal.DemoPortalBase, this_module )
PTKBase.make_ZClass_for_base( ZODBMember.DemoZODBMemberBase, this_module )
PTKBase.make_ZClass_for_base( ZODBMember.DemoZODBMemberFolderBase, this_module )
PTKBase.make_ZClass_for_base( SQLMember.DemoSQLMemberBase, this_module )
PTKBase.make_ZClass_for_base( SQLMember.DemoSQLMemberFolderBase, this_module )

make_and_alias_ZClass_for_base( Document.Document )
make_and_alias_ZClass_for_base( File.File )
make_and_alias_ZClass_for_base( Image.Image )
make_and_alias_ZClass_for_base( Link.Link )
make_and_alias_ZClass_for_base( NewsItem.NewsItem )
make_and_alias_ZClass_for_base( Discussions.Discussable )
make_and_alias_ZClass_for_base( Discussions.DiscussionResponse )

if SUPPLY_DEPRECATED_PTK_BASE_ALIASES:

    #   Get the old module names aliased into sys.modules...
    __module_aliases__ = ( ( 'Products.PTKBase.Document', Document )
                         , ( 'Products.PTKBase.File', File )
                         , ( 'Products.PTKBase.Image', Image )
                         , ( 'Products.PTKBase.Link', Link )
                         , ( 'Products.PTKBase.NewsItem', NewsItem )
                         , ( 'Products.PTKBase.Discussions', Discussions )
                         )

    #   ...and make sure we can find them in PTKBase when we do
    #   'manage_migrate_content()'.
    PTKBase.Document = Document
    PTKBase.File = File
    PTKBase.Image = Image
    PTKBase.Link = Link
    PTKBase.NewsItem = NewsItem

#   Register our stuff in the ProductContect object.
def initialize(context):
    try:
        # Portal classs
        context.registerZClass( _ZClass_for_DemoPortalBase )

        # Membership classes
        context.registerZClass( _ZClass_for_DemoZODBMemberBase
                              , 'Demo ZODB Member Base') 
        context.registerZClass( _ZClass_for_DemoZODBMemberFolderBase
                              , 'Demo ZODB Member Folder Base') 
        context.registerZClass( _ZClass_for_DemoSQLMemberBase
                              , 'Demo SQL Member Base') 
        context.registerZClass( _ZClass_for_DemoSQLMemberFolderBase
                              , 'Demo SQL Member Folder Base')

        # Portal Content types
        context.registerZClass( _ZClass_for_Document
                              , 'Document')
        context.registerZClass( _ZClass_for_Link
                              , 'Link')
        context.registerZClass( _ZClass_for_NewsItem
                              , 'News Item')
        context.registerZClass( _ZClass_for_File
                              , 'PTK File')
        context.registerZClass( _ZClass_for_Image
                              , 'PTK Image')
        context.registerZClass( _ZClass_for_Discussable
                              , 'PTK Discussable mix-in')
        context.registerZClass( _ZClass_for_DiscussionResponse
                              , 'PTK Discussion Response mix-in')
    except:
        print 'Something blew a gasket while initializing PTKBase:'
        import traceback
        traceback.print_exc()
